pro overlay, a, xx, yy, NLEVELS = nlev, TITLES = titles, $
                        WINDOW_SCALE = window_scale 
; NAME:
;	OVERLAY
;
; PURPOSE:
;	Overlay an image and a contour plot.
;
; CATEGORY:
;	General graphics.
;
; CALLING SEQUENCE:
;	OVERLAY, A [, XX, YY]
;
; INPUTS:
;	A:	The two-dimensional array to display.
;	XX,YY:	The fields containing the axis grids.
;
; KEYWORD PARAMETERS:
; NLEVELS: Number of contour levels to be drawn
; TITLES: Four elements array containing the three axis titles
;	and a global title for the plot
; WINDOW_SCALE:	Set this keyword to scale the window size to the image size.
;	Otherwise, the image size is scaled to the window size.
;	This keyword is ignored when outputting to devices with 
;	scalable pixels (e.g., PostScript).
;
; COPYRIGHT:
;	Volker Ossenkopf, September 1999
;-

on_error,2                      ;Return to caller if an error occurs
sz = size(a)			;Size of image
if sz[0] lt 2 then message, 'Parameter not 2D'

	;set window used by contour
contour,[[0,0],[1,1]],/nodata, xstyle=4, ystyle = 4

px = !x.window * !d.x_vsize	;Get size of window in device units
py = !y.window * !d.y_vsize
py = py+20
swx = (px[1]-px[0])*0.83	;Size in x in device units, add space for color bar
swy = py[1]-py[0]		;Size in Y
six = float(sz[1])		;Image sizes
siy = float(sz[2])
aspi = six / siy		;Image aspect ratio
aspw = swx / swy		;Window aspect ratio
f = aspi / aspw			;Ratio of aspect ratios

if (!d.flags and 1) ne 0 then begin	;Scalable pixels?
				;Adjust window size
  if f ge 1.0 then swy = swy / f else swx = swx * f

  tvscl,a,px[0],py[0],xsize = swx, ysize = swy, /device

endif else begin	;Not scalable pixels	
   if keyword_set(window_scale) then begin ;Scale window to image?
	tvscl,a,px[0],py[0]	;Output image
	swx = six		;Set window size from image
	swy = siy
    endif else begin		;Scale window
		if f ge 1.0 then swy = swy / f else swx = swx * f
	tvscl,congrid(bytscl(a),swx,swy),px[0],py[0]
	endelse			;window_scale
     endelse			;scalable pixels

mx = !d.n_colors-1		;Brightest color
				;color vectors
; default
if n_elements(nlev) EQ 0 then nlev=5

if nlev NE 0 then begin
colors=intarr(nlev)
for i=0,(nlev-1)/2 do colors[i]=mx
for i=(nlev-1)/2+1,nlev-1 do colors[i]=0
nodata=0
; Create really equidistant levels
a0=min(a)
da=(max(a)-a0)/(nlev+1)
levels=fltarr(nlev)
for j=0,nlev-1 do levels[j]=a0+(j+1)*da

; No contours
endif else begin
colors=[mx]
nodata=1
levels=[min(a)]
endelse

if n_elements(titles) EQ 0 then titles=[' ',' ',' ']

				;Do the contour
if (n_elements(xx) eq 0) then begin
	   contour,a,/noerase,/xst,/yst,$	
	   pos = [px[0],py[0], px[0]+swx,py[0]+swy],/dev,$
	   levels = levels, c_color =  colors, nodata = nodata, $
           xtitle=titles[0], ytitle=titles[1]
endif else contour,a,xx,yy,/noerase,/xst,/yst,$
	   pos = [px[0],py[0], px[0]+swx,py[0]+swy],/dev,$
	   levels = levels, c_color =  colors, nodata = nodata, $
           xtitle=titles[0], ytitle=titles[1]

; Title
if (n_elements(titles) gt 3) then xyouts, px[0]+0.05*swx, py[0]+0.94*swy, /dev, titles[3], $
	   charsize=!p.charsize*1.2, color=colors[0], charthick=2.0

; Create the color bar
steps = 100
xsize = swx/10
ysize = swy/steps
xpos = px[0] + swx +1
ypos = py[0]

a0=min(a)
da=(max(a)-a0)/(steps-1)
feld2=fltarr(2,steps)
for j=0,steps-1 do begin feld2[0,j]=a0+j*da & feld2[1,j]=feld2[0,j] & endfor

if (!d.flags and 1) ne 0 then begin	;Scalable pixels
  tvscl,feld2,xpos,ypos,xsize = xsize, ysize = swy, /device
endif else begin			;Not scalable pixels	
for j=0,steps-1 do begin
  color=fix((float(!d.table_size)/steps)*j)
  tv, replicate(color,xsize,fix(ysize)+1), fix(xpos), fix(ypos+j*ysize)
endfor
endelse

ax1=[0.0,1.0]
ax2=fltarr(steps)
x0=min(a)
dx=(max(a)-min(a))/steps
for j=0,steps-1 do ax2[j]=x0+(j+0.5)*dx
contour, feld2, ax1,ax2, /noerase, $
         pos=[xpos,ypos,fix(xpos+xsize),ypos+swy], /dev, $
         levels = levels, c_color = colors, nodata = nodata, ystyle=5, $
         xticks=1, xtickname=replicate(' ',2), xminor=1 

axis, yaxis=1, ytitle=titles[2], ystyle=1, ticklen=0.2, yminor=5

return
end
